#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only

import time
import os
import errno
import subprocess as sp

from funnylog2 import logger
from os.path import abspath
from os.path import dirname
from contextlib import contextmanager
from Xlib.display import Display


def recording_screen(name, job_dir=None):
    """
     录制视频
    :param name: 视频名称
    :return:
    """
    logger.info("开始录屏")
    if job_dir is None:
        job_dir = dirname(dirname(abspath(__file__)))
    record_path = (
        f'{job_dir}/report/record/{time.strftime("%Y-%m-%d", time.localtime())}/'
    )
    if not os.path.exists(record_path):
        os.makedirs(record_path)

    _display = Display(os.environ['DISPLAY'])
    width = _display.screen().width_in_pixels
    height = _display.screen().height_in_pixels
    cmd, paths = _create_ffmpeg_cmd(
        width,
        height,
        record_path,
        f"{name.encode('utf-8').decode('unicode-escape')[:40].replace('/', '_')}_autotest",
    )

    for path in paths:
        with suppress(OSError, errnos=(errno.ENOENT, errno.ENAMETOOLONG)):
            os.remove(path)
        logger.info(f"录屏路径存放  {path}")

    with open(os.devnull, "w", encoding="utf-8") as dev_null:
        with sp.Popen(cmd, stdin=sp.PIPE, stdout=dev_null, stderr=dev_null, close_fds=True) as proc:
            time.sleep(0.5)
            if proc.poll() is not None:
                raise RuntimeError("ffmpeg did not start")

            try:
                yield paths[0]
            finally:
                logger.info("停止录屏")
                try:
                    with suppress(IOError, errnos=(errno.EINVAL, errno.EPIPE)):
                        logger.debug(" stop step 1")
                        proc.communicate(input=b"q", timeout=10)
                        logger.debug(f"录屏状态 {proc.stdin.closed}")
                        if not proc.stdin.closed:
                            proc.stdin.close()
                            logger.debug("停止录屏 finish")
                except:
                    try:
                        for _ in range(5):
                            proc.communicate(input=b"q", timeout=10)
                            proc.stdin.close()
                            time.sleep(0.5)
                            if proc.stdin.closed:
                                break
                    except:
                        pass

                logger.info("停止录屏 finish")


def _create_ffmpeg_cmd(width, height, dir_path, file_name, qp=1):
    """
     组装ffmpeg命令行参数
    :param width: 宽
    :param height: 高
    :param dir_path: 视频路径
    :param file_name: 视频名称
    :param qp:
    :return:
    """
    cmd = ["ffmpeg"]

    wh = f"{width}x{height}"
    cmd.extend(
        [
            "-framerate",
            "25",
            "-video_size",
            wh,
            "-f",
            "x11grab",
            "-i",
            f"{os.environ.get('DISPLAY', None)}",
        ]
    )

    paths = []
    file_path = os.path.join(dir_path, file_name + "{}.mp4")
    output_fmt = ["-c:v", "libx264", "-qp", str(qp), "-preset", "ultrafast"]
    path = file_path.format("")
    cmd.extend(output_fmt + [path])
    paths.append(path)
    logger.debug(" ".join(cmd))
    return cmd, paths


@contextmanager
def suppress(exception, errnos):
    """
     抛异常
    :param exception:
    :param errnos:
    :return:
    """
    try:
        yield
    except exception as e:
        # logger.error(f"异常 {e}")
        if errno and e.errno not in errnos:
            logger.debug("step 2")
            raise



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

import sys
import subprocess

from funnylog2 import logger


class CmdCtl:
    """命令行工具"""

    @staticmethod
    def _run(command, _input=None, timeout=None, check=False, **kwargs):
        """run"""
        with subprocess.Popen(command, **kwargs) as process:
            try:
                stdout, stderr = process.communicate(_input, timeout=timeout)
            except:  # Including KeyboardInterrupt, communicate handled that.
                process.kill()
                raise
            ret_code = process.poll()
            if check and ret_code:
                raise subprocess.CalledProcessError(
                    ret_code, process.args, output=stdout, stderr=stderr
                )
        return subprocess.CompletedProcess(process.args, ret_code, stdout, stderr)

    @classmethod
    def _getstatusoutput(cls, command, timeout):
        """getstatusoutput"""
        kwargs = {
            "shell": True,
            "stderr": subprocess.STDOUT,
            "stdout": subprocess.PIPE,
            "timeout": timeout,
        }
        try:
            if sys.version_info >= (3, 7):
                kwargs["text"] = True
            result = cls._run(command, **kwargs
                              )
            data = result.stdout
            if isinstance(data, bytes):
                data = data.decode("utf-8")
            exitcode = result.returncode
        except subprocess.CalledProcessError as ex:
            data = ex.output
            exitcode = ex.returncode
        except subprocess.TimeoutExpired as ex:
            data = ex.__str__()
            exitcode = -1
        if data[-1:] == "\n":
            data = data[:-1]
        return exitcode, data

    @classmethod
    def run_cmd(cls, command, interrupt=False, timeout=25, out_debug_flag=True, command_log=True):
        """
         执行shell命令
        :param command: shell 命令
        :param interrupt: 命令异常时是否中断
        :param timeout: 命令执行超时
        :param out_debug_flag: 命令返回信息输出日志
        :param command_log: 执行的命令字符串日志
        :return: 返回终端输出
        """
        status, out = cls._getstatusoutput(command, timeout=timeout)
        if command_log:
            logger.debug(command)
        if status and interrupt:
            raise f"shell执行失败！| {out}"
        if out_debug_flag and out:
            logger.debug(out)
        return out



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

import allure
import pytest

from os import remove
from os.path import exists
from os.path import splitext
from time import sleep
from funnylog2 import logger
from shutil import copyfile
from datetime import datetime
from pytest_record_video.cmdctl import CmdCtl
from pytest_record_video.recording_screen import recording_screen


def pytest_addoption(parser):
    parser.addoption(
        "--record_video", action="store", default=False,
        help="用例录屏，开启后每条用例都录制视频"
    )
    parser.addoption(
        "--record_failed_video", action="store", default=False,
        help="失败录屏，开启后只有失败用例保存视频"
    )
    parser.addoption(
        "--record_failed_num", action="store", default=False,
        help="失败录屏，从第几次失败开始录制视频，此参数需要结合 pytest-rerunfailures 插件使用"
    )


def pytest_runtest_setup(item):
    # 用例执行前，判断是否开启录屏
    if (item.config.option.record_video
            or item.config.option.record_failed_video
            or item.config.option.record_failed_num):
        item.record = {}
        _job_dir = item.session.fspath.strpath
        _record_name = f'{item.name}_{datetime.now().strftime("%Y-%m-%d_%H-%M-%S")}'

        try:
            if item.config.option.record_video or item.config.option.record_failed_video:
                # 用例录屏
                item.record["object"] = recording_screen(_record_name, _job_dir)
                item.record["image_path"] = next(item.record["object"])
                sleep(3)
            elif item.config.option.record_failed_num:
                # 失败录屏，从第几次失败开始录制视频，此参数需要结合 pytest-rerunfailures 插件使用
                execution_count = getattr(item, "execution_count", False)
                if (execution_count is not False
                        and execution_count > int(item.config.option.record_failed_num)):
                    item.record["object"] = recording_screen(_record_name, _job_dir)
                    item.record["image_path"] = next(item.record["object"])
                    sleep(3)

        except Exception as e:
            logger.info(f"录屏失败: {e}")


@pytest.hookimpl(hookwrapper=True, tryfirst=True)
def pytest_runtest_makereport(item, call):
    out = yield
    report = out.get_result()

    try:
        if (item.config.option.record_video
                or item.config.option.record_failed_video
                or item.config.option.record_failed_num):
            if report.when == "call":  # 存放录屏当次测试结果
                item.record["result"] = report.outcome
                try:
                    # 记录断言的模板图片
                    item.record["template"] = call.excinfo.value.args[0].args[1:]
                except (IndexError, KeyError, AttributeError):
                    # 记录ocr识别区域图片
                    try:
                        pic = call.excinfo.value.args[0][1]
                        if isinstance(pic, tuple):
                            item.record["ocr"] = call.excinfo.value.args[0][1]
                    except (IndexError, AttributeError, TypeError):
                        # 非ocr断言
                        pass
            elif report.when == "teardown":
                try:
                    sleep(3)
                    # 调用生成器保存视频
                    next(item.record["object"])
                except StopIteration:
                    _case_passed = "passed"
                    # 录屏时测试结果为passed，则删除视频
                    _is_remove_record = (item.record.get("result") == _case_passed
                                         and item.config.option.record_video is False)
                    if _is_remove_record:
                        try:
                            remove(item.record["image_path"])
                        except FileNotFoundError:
                            pass
                    else:
                        _screen_cache = "/tmp/screen.png"
                        if exists(_screen_cache):
                            screen_png = f"{splitext(item.record['image_path'])[0]}.png"
                            copyfile(_screen_cache, screen_png)
                            allure.attach.file(
                                screen_png,
                                name="屏幕截图",
                                attachment_type=allure.attachment_type.PNG,
                            )
                            try:
                                for index, tem in enumerate(item.record["template"]):
                                    template = f"{splitext(item.record['image_path'])[0]}_template_{index}.png"
                                    CmdCtl.run_cmd(f"cp {tem}.png {template}")
                                    allure.attach.file(
                                        template,
                                        name="模板图片",
                                        attachment_type=allure.attachment_type.PNG,
                                    )
                            except (FileNotFoundError, KeyError):
                                # 非图像识别错误
                                pass
                            try:
                                template = f"{splitext(item.record['image_path'])[0]}_ocr_.png"
                                CmdCtl.run_cmd(f"cp {item.record['ocr']} {template}")
                                allure.attach.file(
                                    template,
                                    name="OCR识别区域",
                                    attachment_type=allure.attachment_type.PNG,
                                )
                            except KeyError:
                                # ocr 识别区域
                                pass
                        allure.attach.file(
                            item.record["image_path"],
                            name="用例视频",
                            attachment_type=allure.attachment_type.MP4,
                        )
                    logger.info(
                        "结束录屏! "
                        f"{'用例测试通过，删除视频录像' if _is_remove_record else ''}"
                    )
    except (AttributeError, KeyError):
        pass
